//---------------------------------------------------------------------------
// static service for image management
// underlying image lib

#pragma hdrstop

#include "Images.h"

//---------------------------------------------------------------------------
#include "FreeImage.h"
#include "Logger.h"

#pragma package(smart_init)

//---------------------------------------------------------------------------
bool Images::read(Graphics::TBitmap* toBmp, AnsiString filePath){
  bool success = false;

  if(FileExists(filePath)){
    char* chFilePath = strdup(filePath.c_str());
    FREE_IMAGE_FORMAT fif = FreeImage_GetFileType(chFilePath);
    
    if(fif == FIF_UNKNOWN){
      TLogger::log(AnsiString("Not an image: ")+filePath);
    }else{
      //ShowMessage(AnsiString("fif = ")+fif+" PNG = "+FIF_PNG);
      FIBITMAP *dib = FreeImage_Load(FIF_PNG, chFilePath, PNG_DEFAULT);

      if(dib == NULL){
        TLogger::log(AnsiString("Cannot load image:")+filePath);
      }else{
        int ww = FreeImage_GetWidth(dib), hh = FreeImage_GetHeight(dib);

        toBmp->Height = hh;
        toBmp->Width = ww;

        HDC hDC = toBmp->Canvas->Handle;
        SetStretchBltMode(hDC, COLORONCOLOR);
        StretchDIBits(hDC, 0, 0, ww, hh, 0,0,ww,hh, FreeImage_GetBits(dib), FreeImage_GetInfo(dib), DIB_RGB_COLORS, SRCCOPY);

        FreeImage_Unload(dib);
        success = true;
      }
    }
    delete chFilePath;
  }
  return success;
}

//---------------------------------------------------------------------------
Graphics::TBitmap* Images::read(AnsiString filePath){
  Graphics::TBitmap* bmp = new Graphics::TBitmap();
  return read(bmp, filePath) ? bmp : NULL;
}

//---------------------------------------------------------------------------
/*
bool Images::write(AnsiString filePath, Graphics::TBitmap* bmp){
  return write(filePath, bmp, clBlack, clBlack);
}
*/
//---------------------------------------------------------------------------
bool Images::saveFont(AnsiString filePath, Graphics::TBitmap* bmp, bool rotated){
  HBITMAP hbmp = bmp->Handle;
  BITMAP bm;
  GetObject(hbmp, sizeof(BITMAP), (LPSTR) &bm);

  TColor bgColor = bmp->Canvas->Pixels[0][0];
  TColor fgColor;
  FIBITMAP *mono = rotated ?
    FreeImage_Allocate(bm.bmHeight, bm.bmWidth, 1) :
    FreeImage_Allocate(bm.bmWidth, bm.bmHeight, 1);
  
  int w = FreeImage_GetWidth(mono), h = FreeImage_GetHeight(mono);
  for(int y = 0; y < h; y++){
    BYTE* scanLine = FreeImage_GetScanLine(mono, y);
    for(int x = 0; x < w; x++){
      TColor col = rotated ?
        bmp->Canvas->Pixels[h-y-1][w-x-1] :
        bmp->Canvas->Pixels[x][h-y-1];

      if(col != bgColor) {
        int pos = x/8;
        BYTE orMask = 1 << (7 - x%8);
        scanLine[pos] |= orMask;
        fgColor = col;
      }
    }
  }
  RGBQUAD *pal = FreeImage_GetPalette(mono);
  pal[0].rgbRed = GetRValue(bgColor);
  pal[0].rgbGreen = GetGValue(bgColor);
  pal[0].rgbBlue = GetBValue(bgColor);

  pal[1].rgbRed = GetRValue(fgColor);
  pal[1].rgbGreen = GetGValue(fgColor);
  pal[1].rgbBlue = GetBValue(fgColor);

  bool res = FreeImage_Save(FIF_PNG, mono, filePath.c_str());
  FreeImage_Unload(mono);

  if(!res){
    ShowMessage(AnsiString("Can't save ") + filePath);
  }
  return res;

}

//---------------------------------------------------------------------------
bool Images::write(AnsiString filePath, Graphics::TBitmap* bmp){
  HBITMAP hbmp = bmp->Handle;
  BITMAP bm;
  GetObject(hbmp, sizeof(BITMAP), (LPSTR) &bm);
  FIBITMAP *dib = FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);

  HDC dc = GetDC(NULL);
  GetDIBits(dc, hbmp, 0, FreeImage_GetHeight(dib),
    FreeImage_GetBits(dib), FreeImage_GetInfo(dib), DIB_RGB_COLORS);
  ReleaseDC(NULL, dc);

  bool res = FreeImage_Save(FIF_PNG, dib, filePath.c_str());
  FreeImage_Unload(dib);
  if(!res){
    ShowMessage(AnsiString("Can't save ") + filePath);
  }
  return res;
}

//---------------------------------------------------------------------------
TStringList* Images::listImages(TStringList* imageFileList, AnsiString dirPath, bool loadImages){
  imageFileList->Clear();
  
  TSearchRec sr;
  if(FindFirst(dirPath+"\\*.*", faAnyFile, sr) == 0){
    do{
      FREE_IMAGE_FORMAT fif = FreeImage_GetFIFFromFilename(sr.Name.c_str());
      if(fif != FIF_UNKNOWN){
        AnsiString imagePath = dirPath + "\\" + sr.Name;
        Graphics::TBitmap* bmp = read(imagePath);
        if(bmp != NULL){
          if(loadImages){
            imageFileList->AddObject(imagePath, bmp);
          }else{
              imageFileList->Add(imagePath);
          }
        }
      }

    } while (FindNext(sr) == 0);
  }
  FindClose(sr);
  return imageFileList;
}

//---------------------------------------------------------------------------
Graphics::TBitmap* Images::rotated(Graphics::TBitmap* bmp){
  Graphics::TBitmap* transposed = new Graphics::TBitmap();
  transposed->Width = bmp->Height;
  transposed->Height = bmp->Width;
  TCanvas *src = bmp->Canvas, *dst = transposed->Canvas;
  for(int x = 0, w = bmp->Width; x < w; x++){
    for(int y = 0, h =  bmp->Height; y < h; y++){
      dst->Pixels[h - y][x] = src->Pixels[x][y];
    }
  }

  return transposed;
}
