//---------------------------------------------------------------------------

#include <vcl.h>
#include <ComCtrls.hpp>
#include <Buttons.hpp>
#include <StrUtils.hpp>

#pragma hdrstop

#include "Localizer.h"
#include "Logger.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

AnsiString TLocalizer::LOCALE_ID;
AnsiString LOC_FILE_HEADER =
"// This is a localization file for '${LANG_NAME}' language, for the TCBR shell\r\n\
// To localize the shell, you may translate the messages below \r\n\
// and rename this file to ${MSG_FILE}\r\n\
\r\n\
// The line below (AUTHOR_MODE entry) enables useful localization-related diagnostic messages\r\n\
// remove it as you finish localization\r\n\r\n\
AUTHOR_MODE=on\r\n\r\n\
// if you localize the shell for your language, please consider contributing\r\n\
//  - send it to me and you'll be credited each time the shell runs in your language:\r\n\r\n\
";

// AUTHOR_NAME=Type your name here

void replace(AnsiString& where, AnsiString what, AnsiString with){
  int pos;
  while((pos = where.Pos(what)) > 0){
    where.Delete(pos, what.Length());
    where.Insert(with, pos);
  }
} 

// run by:  EnumSystemLocales((LOCALE_ENUMPROC)EnumLocalesProc, LCID_SUPPORTED);
BOOL __stdcall EnumLocalesProc(char* lpLocaleString){
  AnsiString LocaleName, LanguageName, CountryName;
  LCID lcid;
  lcid = StrToInt("$" + AnsiString(lpLocaleString));
  LocaleName = GetLocaleStr(lcid, LOCALE_SABBREVLANGNAME, "");
  LanguageName = GetLocaleStr(lcid, LOCALE_SNATIVELANGNAME, "");
  CountryName = GetLocaleStr(lcid, LOCALE_SNATIVECTRYNAME, "");
  if (lstrlen(LocaleName.c_str()) > 0)
    TLogger::log(LocaleName + ":" + LanguageName + "-" + CountryName);

  return TRUE;
}

//----------------------------------------------------------
AnsiString getLabelProperty(TComponent* compo){
  AnsiString name = "";
  if(compo->InheritsFrom(__classid( TToolBar ))) name = "";
  else
  if(compo->InheritsFrom(__classid( TLabel ))) name = "Caption";
  else
  if(compo->InheritsFrom(__classid( TButtonControl ))) name = "Caption";
  else
  if(compo->InheritsFrom(__classid( TGraphicControl ))) name = "Hint";
  else
  if(compo->InheritsFrom(__classid( TControl ))) name = "Caption";
  
  if(IsPublishedProp(compo, name)) return name;
  return "";
}

//---------------------------------------------------------------------------
AnsiString getCaption(TComponent* compo){
  AnsiString name = getLabelProperty(compo);
  return (name != "") ? GetStrProp(compo, name): AnsiString("");
}

//---------------------------------------------------------------------------
void setCaption(TComponent* compo, AnsiString value){
  AnsiString name = getLabelProperty(compo);
  if(name != ""){
    SetStrProp(compo, name, value);
  }
}


//---------------------------------------------------------------------------
bool TLocalizer::AUTHOR_MODE = false;
TStringList* TLocalizer::MESSAGES = NULL;
TStringList* TLocalizer::LOCALIZATION_ENTRIES = NULL;

/*
AnsiString* registerMessage(char* id, char* text){
  AnsiString* str = new AnsiString();
  *str+= text;
  return str;
}
*/
// #define MSG(id, text) AnsiString& id = *registerMessage(#id, text);

// MSG(MSG_FIRST, "first")
// MSG(MSG_SECOND, "second")


//---------------------------------------------------------------------------
void TLocalizer::init(){
  if(MESSAGES != NULL) return;

  // EnumSystemLocales((LOCALE_ENUMPROC)EnumLocalesProc, LCID_SUPPORTED);

  MESSAGES = new TStringList();

  LCID lcid = GetUserDefaultLangID();
  LOCALE_ID = GetLocaleStr(lcid, LOCALE_SABBREVLANGNAME, "");

//  LOCALE_ID = "hun";

  AnsiString localMsg = getLocalFile("shellmsg.txt");

  if(FileExists(localMsg)){
    MESSAGES->LoadFromFile(localMsg);
    //AUTHOR_MODE = (MESSAGES->IndexOf("AUTHOR_MODE=on") >= 0);
    AUTHOR_MODE = (findEntry("AUTHOR_MODE") == "on");

    //AnsiString authorName = findEntry("AUTHOR_NAME");
    //if(authorName != ""){
      //TLogger::logHeader(AnsiString("Localizaton author: ") + authorName, false);
    //}
  }else{
    AnsiString templateLocalMsg = getLocalFile("template-shellmsg.txt");
    if(!FileExists(templateLocalMsg)){
      TLogger::log(AnsiString("There is no localization for your language yet:(\n")+
        "Become co-author of bookreader - translate it into your language!\n"+
        "The localization template has been created: "+templateLocalMsg
        +"\nRefer to its contents and don't hesitate to ask tequilacat for help if needed");


      AnsiString LocaleName = GetLocaleStr(lcid, LOCALE_SABBREVLANGNAME, "");
      AnsiString LanguageName = GetLocaleStr(lcid, LOCALE_SNATIVELANGNAME, "");
      AnsiString CountryName = GetLocaleStr(lcid, LOCALE_SNATIVECTRYNAME, "");

      replace(LOC_FILE_HEADER, "${MSG_FILE}", AnsiString("shellmsg-")+LOCALE_ID + ".txt");
      replace(LOC_FILE_HEADER, "${LANG_NAME}", LanguageName);

      LOCALIZATION_ENTRIES = new TStringList();
      LOCALIZATION_ENTRIES->Add(LOC_FILE_HEADER);

//      TLogger::log(AnsiString("Create ") +LanguageName+" localization file: "+templateLocalMsg);
    }
  }
}

//---------------------------------------------------------------------------
// format:
// COMPONAME=.....
void TLocalizer::localize(TForm* form){
  init();

  TStringList* nonLocalizedComponents = new TStringList();
  if(LOCALIZATION_ENTRIES != NULL && form != Application->MainForm){
    LOCALIZATION_ENTRIES->Add(AnsiString("\n// Localize Form ") +form->Name+ " '"+ form->Caption +"'\n");
    LOCALIZATION_ENTRIES->Add(form->Name+" = "+ form->Caption+" // form caption\n");
  }

  for(int i = 0, n = form->ComponentCount; i < n; i++){
    TComponent* compo = form->Components[i];
    AnsiString caption = getCaption(compo);
    if(caption != "" && compo->Name.SubString(1,1)!='_'){
      nonLocalizedComponents->Add(compo->Name);
      if(LOCALIZATION_ENTRIES != NULL){
        AnsiString entry = form->Name +"." + compo->Name + " = " +caption+" // " + compo->ClassName();
        LOCALIZATION_ENTRIES->Add(entry);
      }
    }
  }

  AnsiString formNameMarker = form->Name+".";
  for(int i = 0; i < MESSAGES->Count; i++){
    AnsiString name, val;
    if(!parse(MESSAGES->Strings[i], name, val))continue;
    
    /*AnsiString str = MESSAGES->Strings[i];
    int pos = str.Pos("//");
    if(pos > 0) str = str.SubString(1, pos-1);
    pos = str.Pos('=');
    if(pos == 0) continue;
    AnsiString name = str.SubString(1, pos-1).Trim(), val = str.SubString(pos+1, str.Length()).Trim();
    */

    if(name == form->Name){
      form->Caption = val;
    }else
    if(name.Pos(formNameMarker) == 1 && val != ""){
      name = name.SubString(formNameMarker.Length()+1, name.Length());
      TComponent* compo = form->FindComponent(name);

      if(compo == NULL){
        if(AUTHOR_MODE){
          TLogger::error(AnsiString("No component: ")+name+" = '"+val+"'");
        }
      }else{
        int nlPos = nonLocalizedComponents->IndexOf(compo->Name);
        if(nlPos != -1){
          nonLocalizedComponents->Delete(nlPos);
        }
        if(AUTHOR_MODE && getLabelProperty(compo) == ""){
          TLogger::error(AnsiString("Component ") + compo->Name +" ("
            + compo->ClassName() +") cannot be localized");
        }
        setCaption(compo, val);
      }
    }
  }


  if(AUTHOR_MODE && nonLocalizedComponents->Count > 0){
    TLogger::error("Not localized components: ");
    for(int i = 0; i < nonLocalizedComponents->Count; i++){
      AnsiString name = nonLocalizedComponents->Strings[i];
      TComponent* compo = form->FindComponent(name);
      TLogger::error(formNameMarker + name + " = "+ getCaption(compo) + " // " + compo->ClassName());
    }
  }

  if(LOCALIZATION_ENTRIES != NULL){
    AnsiString templateLocalMsg = getLocalFile("template-shellmsg.txt");
    TFileStream* fOut = new TFileStream(templateLocalMsg, fmCreate|fmShareDenyNone);
    LOCALIZATION_ENTRIES->SaveToStream(fOut);
    delete fOut;
  }
}

AnsiString TLocalizer::localMessage(char* lngMessageId){
  return "";
}

AnsiString LNG(char* lngMessageId, AnsiString defaultText){
  return defaultText; // TLocalizer::localMessage(lngMessageId);
}

////////////////////////////////////////////////////////////////////////////
AnsiString TLocalizer::getLocalFile(AnsiString fileName){
  AnsiString localFileName = fileName;
  int pos = fileName.LastDelimiter('.');

  if(pos != 0) {
    localFileName = fileName.SubString(1, pos-1) + "-"+ LOCALE_ID + fileName.SubString(pos, fileName.Length());
  }
//  AnsiString langFile = ExtractFileDir(Application->ExeName) + "\\i18n\\" + LOCALE_ID + "-" + fileName;
    AnsiString langFile = ExtractFileDir(Application->ExeName) + "\\i18n\\" + localFileName;
    if(FileExists(langFile)){
    return langFile;
  }
  langFile = ExtractFileDir(Application->ExeName) + "\\i18n\\" + fileName;
//  if(!FileExists(langFile)) TLogger::log(AnsiString("Cannot find any version of ")+fileName);
  return langFile;
}


  /*char *localeString =new char[10];
  int cbBufSize = GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LC_ALL, localeString, 10);
  if(cbBufSize != 0){
    delete localeString;
    localeString =new char[cbBufSize];
    if(GetLocaleInfo(LOCALE_USER_DEFAULT, LC_ALL, localeString, cbBufSize)){
      TLogger::log(AnsiString( "Locale: ") + localeString);
    }
    delete localeString;
  }else{
    TLogger::log(AnsiString( "Error: ") + GetLastError());
    LPVOID lpMsgBuf;

    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        GetLastError(),
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
        (LPTSTR) &lpMsgBuf,
        0,
        NULL
    );

    // Display the string.
    MessageBox( NULL, (const char*)lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );

    // Free the buffer.
    LocalFree( lpMsgBuf );

  }
  */

bool TLocalizer::parse(AnsiString entry, AnsiString& name, AnsiString& val){
    int pos = entry.Pos("//");
    if(pos > 0) entry = entry.SubString(1, pos-1);
    pos = entry.Pos('=');
    if(pos == 0) return false;

    name = entry.SubString(1, pos-1).Trim();
    val = entry.SubString(pos+1, entry.Length()).Trim();
    return true;
}

AnsiString TLocalizer::findEntry(AnsiString key){
  for(int i=0, len = MESSAGES->Count; i < len; i++){
    AnsiString name, val;
    if(parse(MESSAGES->Strings[i], name, val) && name == key){
      return val;
    }
  }

  return "";
}


