//---------------------------------------------------------------------------
//#include <idglobal.hpp>
#include <StrUtils.hpp>
#include <sys\stat.h>

#include <strstream>


#pragma hdrstop

#include "Localizer.h"
#include "Logger.h"
#include "Packer.h"
#include "Images.h"
#include "Utils.h"
#include "FreeImage.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

Packer::Packer(TZipBuilder* theZipBuilder, AnsiString jarRepository):myZipBuilder(theZipBuilder){
//  myChapterDividePatterns = new char[1];
//  *myChapterDividePatterns = 0;
  
    myMidletProperties = new TStringList();
    myManifestContent = new TStringList();
    myEncodingTable = new char[258];
    myCreatedJars = new TStringList();
    myFilePaths = new TStringList();
    myFileNames = new TStringList();

    //myChapterDividePatterns = new TStringList();

    int MaxPath = 400;
    char* chTmpPath = new char[MaxPath+1];
    int len = GetTempPath(MaxPath, chTmpPath);
    if(len > MaxPath){
      delete chTmpPath;
      MaxPath = len;
      chTmpPath = new char[MaxPath+1];
      GetTempPath(MaxPath, chTmpPath);
    }
    myTempPath = chTmpPath;
    if(myTempPath.SubString(myTempPath.Length(), 1) == '\\'){
      myTempPath.SetLength(myTempPath.Length()-1);
    }
    delete chTmpPath;

    EncodingFile = "";
    
    myJarRepository = jarRepository;
    readAvailableJars();
}

int Packer::addText(AnsiString filePath)
{
  //TODO: Add your source code here
  if(FileExists(filePath)){
    AnsiString fileName = ExtractFileName(filePath);
    int lastDotPos = fileName.LastDelimiter(".");
    if(lastDotPos != 0){
      fileName = fileName.SubString(1, lastDotPos-1);
    }
    myFilePaths->Add(filePath);
    myFileNames->Add(fileName);
  }
  return myFilePaths->Count;
}

void Packer::removeText(int index){
  myFilePaths->Delete(index);
  myFileNames->Delete(index);
}

AnsiString Packer::getTextName(int index){
  return myFileNames->Strings[index];
}

AnsiString Packer::getTextPath(int index){
  return myFilePaths->Strings[index];
}

//---------------------------------------------------------------------------
int Packer::getTextCount(){
  return myFilePaths->Count;
}

//---------------------------------------------------------------------------
void Packer::cleanUp(AnsiString mask){
  TSearchRec sr;
  mask = myProjectDir + "\\" + mask;
  if(FindFirst(mask, 0, sr) == 0){
    do{
      //TLogger::log(ShowMessage(AnsiString("Remove: ")+sr.Name);
      DeleteFile(myProjectDir + "\\" + sr.Name);
    } while (FindNext(sr) == 0);
  }
  FindClose(sr);
}

void Packer::setTextName(int index, AnsiString newName){
  if(index >=0 && index < myFileNames->Count){
    myFileNames->Strings[index] = newName;
  }
}

TStringList* Packer::getCreatedJars(){
  return myCreatedJars;
}

//---------------------------------------------------------------------------
// returns set of properties to pass to midlet (in UC). 
void Packer::setMidletProperty(AnsiString name, AnsiString value){
  setManifestField(myMidletProperties, name, value);
}

//---------------------------------------------------------------------------
AnsiString Packer::getMidletProperty(AnsiString name){
  return getManifestField(myMidletProperties, name);
}

//---------------------------------------------------------------------------
void Packer::clearMidletProperties(){
  myMidletProperties->Clear();
}

//---------------------------------------------------------------------------
char* Packer::toUnicode(AnsiString nativeContents){
  char* nativeChars = strdup(nativeContents.c_str());
  char* ucContent = new char[nativeContents.Length()*2];
  char* ucPtr = ucContent;

  for(int i = 0; i < nativeContents.Length(); i++){
    unsigned char ch = nativeChars[i]; //nativeContents[i+1];
    if(myEncodingFile == "" || ch < 128) {
      *ucPtr++ = 0;
      *ucPtr++ = ch;
    }else{
      int index = 2 + (ch - 128)*2;
      *ucPtr++ = myEncodingTable[index++]; *ucPtr++ = myEncodingTable[index];
    }
  }

  delete nativeChars;
  return ucContent;
}

//---------------------------------------------------------------------------
// writes properties in unicode, in format of BR RS Sring Dump
bool Packer::writeMidletProperties(AnsiString jarFile){
//  TLogger::log("write props");
  if(LocalizationFile != ""){
     myZipBuilder->ZipStream->Size = 0;
     myZipBuilder->ZipStream->LoadFromFile(LocalizationFile);
     myZipBuilder->AddStreamToFile("loc.txt");
     /*
     int fIn = FileOpen(myEncodingFile, fmOpenRead | fmShareDenyNone);
     if(fIn < 0){
        TLogger::error(AnsiString("Cannot open  file ")+myEncodingFile);
     }else{

        int size =
        FileRead(fIn, , 258);
        myZipBuilder->ZipStream->Write(, );
        FileClose(fIn);
        myZipBuilder->AddStreamToFile("loc.txt");
     }
     */

  }

  myZipBuilder->ZipStream->Size = 0;

  char ucHeader[2];
  ucHeader[0] = 0xfe;
  ucHeader[1] = 0xff;
  myZipBuilder->ZipStream->Write(ucHeader, 2);

  for(int i = 0; i < myMidletProperties->Count; i++){
    AnsiString native = myMidletProperties->Strings[i]+"\n";
    int pos = native.Pos(": ");
    native = native.SubString(1, pos-1) + "="+ native.SubString(pos+2, native.Length());
    char* ucContent = toUnicode(native);
    myZipBuilder->ZipStream->Write(ucContent, native.Length() * 2);
    delete ucContent;
    // convert from "name: value" to "name=value\n",
    // then convert to Unicode,
    // then write to ZIP
  }
//  myZipBuilder->ZipStream->Write(char*, buflen);
//  myZipBuilder->ZipStream->SaveToFile(ExtractFileDir(Application->ExeName) + "\\enc.txt");

  return !myZipBuilder->AddStreamToFile("props.ini");
}

//---------------------------------------------------------------------------
// returns position at the end of last phrase (after dot), or upTo if not found.
int cutAtLastPhrase(char* text, int from, int upTo){
  int cutAt = upTo;
  while(--cutAt > from){
    if(text[cutAt]=='.'){
      cutAt++; break;
    }
  }
  return (cutAt == from) ? upTo : cutAt;
}

//---------------------------------------------------------------------------
bool Packer::build(AnsiString dir, AnsiString projectId, int maxBookSize, int maxJarTextSize){
  myMaxBookSize = maxBookSize;
  myProjectDir = dir;
  myMaxJarTextSize = maxJarTextSize;

  if(getTextCount() == 0){
    return False;
  }

  cleanUp(projectId+"*.*");
  
  myDefaultMidletPropertiesCount = myMidletProperties->Count;

  // reset to initialize jar in TMP dir
  myTemplateJarPath = "";
  myCreatedJars->Clear();

  char* fileText = NULL;
  int curFilePosition, curFileIndex = 0;
  int curFileSize;

  AnsiString curJar = "";
  int fileInJar = 0; // valid are 0..(M-1)
  int textFragmentIndex = 0;

  while(curFileIndex < getTextCount()){
    // prepare JAR
    if(curJar == ""){
      curJar = createNextJar(projectId);
      fileInJar = 0;
    }

    // prepare TEXT
    if(fileText == NULL){
      AnsiString filePath = getTextPath(curFileIndex);
      curFileSize = Utils::getFileSize(filePath);
      curFilePosition = 0;
      // read whole file
      fileText = new char[curFileSize];
      textFragmentIndex = 1;

      int fIn = FileOpen(filePath, fmOpenRead | fmShareDenyNone);
      if(fIn < 0){
        ShowMessage(AnsiString("Cannot open input file ")+filePath);
        return false;
      }

      FileRead(fIn, fileText, curFileSize);
      FileClose(fIn);

      myTocBuilder.build(fileText, curFileSize, myChapterDividePatterns.c_str(), false);
    }

    // search chapters, if needed
//    int cutAt = curFileSize;
    int cutAt = myTocBuilder.getNext(curFilePosition);

    if(myMaxBookSize > 0 && myMaxBookSize < cutAt - curFilePosition){
      cutAt = cutAtLastPhrase(fileText, curFilePosition, curFilePosition + myMaxBookSize);
    }

    // now, try to respect JAR size if given.
    // check file size difference, estimate
    AnsiString bookLabel = getTextName(curFileIndex);
    if(textFragmentIndex > 1) bookLabel += AnsiString("(")+textFragmentIndex+")";

    if(myMaxJarTextSize > 0){// binary search ?
      int maxJarSize = myMaxJarTextSize;
      addFileFragment(fileInJar, fileText, curFilePosition, cutAt-curFilePosition, bookLabel);

      int jarSize = Utils::getFileSize(curJar);
      bool hasExceeded = false;
      while(jarSize > maxJarSize){
        hasExceeded = true;
        // find new size to add, descreasing cutAt
//        cutAt = curFilePosition + (cutAt-curFilePosition) * 9 / 10;
        cutAt = cutAtLastPhrase(fileText, curFilePosition, curFilePosition + (cutAt-curFilePosition) * 9 / 10);
        if(cutAt == curFilePosition) {
          // can't keep in size with this file,
          // remove the file from Jar, and put into next one.
          // SHOULD DELETE !!!
          hasExceeded = false;
          break;
        }
        addFileFragment(fileInJar, fileText, curFilePosition, cutAt-curFilePosition, bookLabel);
        jarSize = Utils::getFileSize(curJar);
      }

      // if jar is full, reset curJar, and increment curJarIndex
      if(hasExceeded){
        closeJar(curJar);
        curJar = "";
      }
    }else
      addFileFragment(fileInJar, fileText, curFilePosition, cutAt - curFilePosition, bookLabel);

    if(cutAt > curFilePosition ){
      fileInJar++; // has really added
      textFragmentIndex++;
    }

    curFilePosition  = cutAt;

    if(curFilePosition == curFileSize){
      delete fileText;
      fileText = NULL;
      curFileIndex ++;
    }
  }

  closeJar(curJar);

  return true;
}

//---------------------------------------------------------------------------
// completes Jar by adding book id as midlet name, writing properties
//   and creating JAD file
void Packer::closeJar(AnsiString jarFile){
  AnsiString bookId = ExtractFileName(jarFile);
  bookId = bookId.SubString(1, bookId.Length() - 4);

  AnsiString desc = getManifestField(myManifestContent, "MIDlet-1");
  int pos;
  while((pos = desc.Pos(',')) != 0){
    desc = desc.SubString(pos+1, desc.Length());
  }
  setManifestField(myManifestContent, "MIDlet-1", bookId + ", ,"+desc);
  setManifestField(myManifestContent, "MIDlet-Name", bookId);

  writeMidletProperties(jarFile);

  while(myMidletProperties->Count > myDefaultMidletPropertiesCount)
    myMidletProperties->Delete(myMidletProperties->Count-1);

  myZipBuilder->ZipStream->Size = 0;
  myZipBuilder->ZipStream->Write(myManifestContent->Text.c_str(), myManifestContent->Text.Length());
  myZipBuilder->AddOptions = myZipBuilder->AddOptions << AddRecurseDirs << AddDirNames;
  if(myZipBuilder->AddStreamToFile("META-INF\\manifest.mf")){
    myZipBuilder->AddOptions.Clear();
    TLogger::error("Can't create manifest");
    return;
  }
  myZipBuilder->AddOptions.Clear();

  //TODO: write properties

  myManifestContent->Add(AnsiString("MIDlet-Jar-Size: ")+Utils::getFileSize(jarFile));
  myManifestContent->Add(AnsiString("MIDlet-Jar-URL: ")+bookId+".jar");

  myManifestContent->SaveToFile(ChangeFileExt(jarFile, ".jad"));
  myManifestContent->Delete(myManifestContent->Count-1);
  myManifestContent->Delete(myManifestContent->Count-1);
}

//---------------------------------------------------------------------------
AnsiString Packer::createNextJar(AnsiString suiteId){
  if(myTemplateJarPath == "" || !FileExists(myTemplateJarPath)){
    readManifest(myJarRepository+"\\"+PlatformId, myManifestContent);

    myLastJarNumber = 0;  
    AnsiString tmpFile = myTempPath+"\\bookreader.jar";
    if(!CopyFile((myJarRepository+"\\"+PlatformId).c_str(), tmpFile.c_str(), false)){
      ShowMessage(LNG("LNG_COPY_FAILED_TO", "Cannot copy jar file to ") + tmpFile);
      return "";
    }

    myZipBuilder->ZipFilename = tmpFile;

    myZipBuilder->FSpecArgs->Clear();
    myZipBuilder->FSpecArgs->Add( "textfile*.txt");
    myZipBuilder->FSpecArgs->Add( "font*.png");
    myZipBuilder->FSpecArgs->Add( "encoding.txt");

    if(myZipBuilder->Delete()){
      return "";
    }

    myTemplateJarPath = tmpFile;

    myZipBuilder->FSpecArgs->Clear();
    if(NormalFontImage != NULL){
      Images::saveFont(myTempPath + "\\font.png", NormalFontImage, false);
      myZipBuilder->FSpecArgs->Add(myTempPath + "\\font.png");
    }

    if(RotatedFontImage != NULL){
      Images::saveFont(myTempPath + "\\font-lscp.png", RotatedFontImage, true);
      myZipBuilder->FSpecArgs->Add(myTempPath + "\\font-lscp.png");
    }

    if(myZipBuilder->FSpecArgs->Count > 0){
      myZipBuilder->Add();
    }

    if(EncodingFile != ""){
      myZipBuilder->ZipStream->Size = 0;
      myZipBuilder->ZipStream->Write(myEncodingTable, 258);
      myZipBuilder->AddStreamToFile("encoding.txt");
      //CopyFile(EncodingFile.c_str(), (myTempPath+"\\encoding.txt").c_str(), false);
      //myZipBuilder->FSpecArgs->Add(myTempPath+"\\encoding.txt");
    }

  }

  myLastJarNumber ++;
  AnsiString jarFile = myProjectDir+"\\"+ suiteId;
  if(myLastJarNumber > 1)
    jarFile += AnsiString("_") + myLastJarNumber;
  jarFile += ".jar";
  
  if(!CopyFile(myTemplateJarPath.c_str(), jarFile.c_str(), false)){
    ShowMessage(LNG("LNG_COPY_FAILED_TO", "Cannot copy jar file to ") + jarFile);
    return "";
  }

  myZipBuilder->ZipFilename = jarFile;
  myCreatedJars->Add(jarFile);
//  TLogger::log(AnsiString("Create JAR: ")+suiteId+"_" + myLastJarNumber + ".jar");

  TLogger::log(AnsiString("Create JAR: ")+ExtractFileName(jarFile));

  return jarFile;
}

//---------------------------------------------------------------------------
bool Packer::createJar(AnsiString suiteId, int& textIndex, int& textPosition){
/*
  while(myManifestContent->Count > myManifestFixedCount)
    myManifestContent->Delete(myManifestContent->Count-1);


  AnsiString jarFile = createNextJar(suiteId);
  if(jarFile == "") return false;
  
  int suiteTextSize = 0, booksInJar = 0;
  while(textIndex < getTextCount() && (myMaxJarTextSize == 0 || suiteTextSize < myMaxJarTextSize)){
    // add files (entire or by parts) till size allows
    int fileSize = Utils::getFileSize(getTextPath(textIndex));
    int remains = fileSize - textPosition;
    int chunk = (myMaxBookSize == 0 || myMaxBookSize > remains) ? remains : myMaxBookSize;
    if(myMaxJarTextSize > 0 && suiteTextSize + chunk > myMaxJarTextSize)
      chunk = myMaxJarTextSize - suiteTextSize;

    if(!addFileFragment(booksInJar, getTextPath(textIndex), textPosition, chunk)){
      return false;
    }
    
    AnsiString label = AnsiString("TC-Booklet-Label-")+ booksInJar + ": "+getTextName(textIndex);
    if(chunk != fileSize){
      label += AnsiString(" (")+myCurChapterIndex+")";
    }
    myManifestContent->Add(label);

    booksInJar++;
    myCurChapterIndex++;


    textPosition += chunk;
    suiteTextSize += chunk;
    if(textPosition >= fileSize) {
      textIndex ++;
      textPosition = 0;
      myCurChapterIndex++;
    }
  }
  
  AnsiString suitLabel = suiteId; // (getTextCount() == 1) ? getTextName(0): suiteId;
  myManifestContent->Strings[0] = AnsiString("MIDlet-Name: ")+suiteId;

  for(int i = 0; i < myManifestContent->Count; i++){
    AnsiString entry = myManifestContent->Strings[i];
    if(AnsiStartsStr("MIDlet-1:", entry)){
      int pos = entry.Pos(",");
      myManifestContent->Strings[i] = AnsiString("MIDlet-1: ")+
        suitLabel +entry.SubString(pos, entry.Length() - pos+1);
    }
  }
  
  myZipBuilder->ZipStream->Size = 0;
  myZipBuilder->ZipStream->Write(myManifestContent->Text.c_str(), myManifestContent->Text.Length());
  myZipBuilder->AddOptions = myZipBuilder->AddOptions << AddRecurseDirs << AddDirNames;
  if(myZipBuilder->AddStreamToFile("META-INF\\manifest.mf")){
    return false;
  }
  myZipBuilder->AddOptions.Clear();

  myManifestContent->Add(AnsiString("MIDlet-Jar-Size: ")+Utils::getFileSize(jarFile));
  myManifestContent->Add(AnsiString("MIDlet-Jar-URL: ")+suiteId+".jar");
  myManifestContent->SaveToFile(suiteId+".jad");

  myCreatedJars->Add(jarFile);
  */
  return true;
}

///////////////////////////////////////////////////////////////////
bool Packer::addFileFragment(int numInJar, char* buffer, int from, int size, AnsiString fragmentLabel){
  myZipBuilder->ZipStream->Size = 0;
  myZipBuilder->ZipStream->Write(buffer+from, size);

  AnsiString entryName = (numInJar == 0) ? AnsiString("textfile.txt") : (AnsiString("textfile")+numInJar+".txt");
//  TLogger::log(AnsiString("Add book ")+fragmentLabel+" to "+myZipBuilder->ZipFileName);
  setMidletProperty(AnsiString("J/")+entryName+".label", fragmentLabel);

  return !myZipBuilder->AddStreamToFile(entryName);
}

///////////////////////////////////////////////////////////////////
/*
bool Packer::addFileFragment(int numInJar, AnsiString file, int from, int size){
  int fIn = FileOpen(file, fmOpenRead | fmShareDenyNone);
  if(fIn < 0){
    ShowMessage(AnsiString("Cannot open input file ")+file);
    return false;
  }

  if(size == 0) {
    size = Utils::getFileSize(file);
  }

  char* buffer = new char[size];
  FileSeek(fIn, from, 0);
  int actualRead = FileRead(fIn, buffer, size);
  FileClose(fIn);
  return addFileFragment(numInJar, buffer, 0, actualRead);
}
*/

//================================================================
// reads available jars, records their filenames and descriptions
// entry : BR-VersionDescription
void Packer::readAvailableJars(){
  TSearchRec sr;
  mySourceJarDescriptions = new TStringList();
  mySourceJarVersions = new TStringList();
  TStringList* manifest = new TStringList();

  if(FindFirst(myJarRepository+"\\*.jar", faAnyFile, sr) == 0){
    // extract description from the file, prepares hreadable description
    do {
      AnsiString jarPath = myJarRepository + "\\" + sr.Name;
      readManifest(jarPath, manifest);

      AnsiString version = getManifestField(manifest, "MIDlet-Version");
      AnsiString platform = getManifestField(manifest, "TCBR-Platform");
      if(platform == "") platform = sr.Name;

      AnsiString description = platform + "  (midlet v"+ version +")";

      mySourceJarDescriptions->Add(sr.Name);
      mySourceJarVersions->Add(version);
      mySourceJarDescriptions->Add(description);

    } while (FindNext(sr) == 0);
  }
  FindClose(sr);
  delete manifest;

  mySelectedPlatformIndex = (mySourceJarDescriptions->Count > 0) ? 0:-1;
}

//---------------------------------------------------------
void __fastcall Packer::SetPlatformId(AnsiString value){
  for(int i = 0; i < mySourceJarDescriptions->Count; i+=2){
    if(mySourceJarDescriptions->Strings[i] == value) {
      PlatformIndex = i/2;
      break;
    }
  }
}

//---------------------------------------------------------
AnsiString __fastcall Packer::GetPlatformId(){
  return mySourceJarDescriptions->Strings[mySelectedPlatformIndex*2];
}

//---------------------------------------------------------
void __fastcall Packer::SetPlatformIndex(int value){
  if(value >= 0 && value < mySourceJarDescriptions->Count/2){
    mySelectedPlatformIndex = value;
  }
}

//---------------------------------------------------------
int __fastcall Packer::GetPlatformIndex(){
  return mySelectedPlatformIndex;
}

//---------------------------------------------------------
int __fastcall Packer::GetSupportedPlatformsCount(){
  return mySourceJarDescriptions->Count / 2;
}

//---------------------------------------------------------
AnsiString __fastcall Packer::GetSupportedPlatformVersion(int index){
  return mySourceJarVersions->Strings[index];
}

//---------------------------------------------------------
AnsiString __fastcall Packer::GetSupportedPlatformId(int index){
  return mySourceJarDescriptions->Strings[index + index];
}

//---------------------------------------------------------
AnsiString __fastcall Packer::GetSupportedPlatformDescription(int index){
  return mySourceJarDescriptions->Strings[index + index + 1];
}

//---------------------------------------------------------
TStringList* Packer::readManifest(AnsiString jarFilePath, TStringList* manifest){
  if(manifest == NULL){
    manifest = new TStringList();
  }else
    manifest->Clear();
  AnsiString mftFile = myTempPath +"\\manifest.mf";
  DeleteFile(mftFile);
  myZipBuilder->ExtrBaseDir = myTempPath;
  myZipBuilder->ZipFilename = jarFilePath;
  myZipBuilder->FSpecArgs->Clear();
  myZipBuilder->FSpecArgs->Add( "META-INF\\manifest.mf");

  int errCode = myZipBuilder->Extract();
  manifest->LoadFromFile(mftFile);
  DeleteFile(mftFile);

  // cut empty lines
  for(int i=0; i < manifest->Count; i++){
    if(manifest->Strings[i] == ""){
      manifest->Delete(i);
      i--;
    }
  }

  /*
  this code somehow does not work with JAR generated archives
  althought works with pkzip25-originated ones
  TZipStream * mftStream = myZipBuilder->ExtractFileToStream("META-INF\\manifest.mf");
  if(mftStream != NULL)
    manifest->LoadFromStream(mftStream);
    */

  return manifest;
}

//---------------------------------------------------------
void Packer::setManifestField(TStringList* manifest, AnsiString fieldName, AnsiString value){
  for(int i = 0; i < manifest->Count; i ++){
    AnsiString curEntry = manifest->Strings[i];
    if(curEntry.Pos(fieldName+":") == 1){
      manifest->Strings[i] = fieldName + ": "+value;
      return;
    }
  }
  manifest->Add(fieldName + ": "+value);
}

//---------------------------------------------------------
AnsiString Packer::getManifestField(TStringList* manifest, AnsiString fieldName){
  for(int i = 0; i < manifest->Count; i ++){
    AnsiString curEntry = manifest->Strings[i];
    if(curEntry.Pos(fieldName+":") == 1){
      return curEntry.SubString(fieldName.Length()+2, curEntry.Length() - fieldName.Length()-1).Trim();
    }
  }

  return "";
}

//---------------------------------------------------------
/*
int Packer::findNextChapter(char* fileText, int from, int fileSize){
  return from;
}
*/
//---------------------------------------------------------
void __fastcall Packer::SetChapterPatterns(AnsiString value){
  myChapterDividePatterns = value;
}

//---------------------------------------------------------
AnsiString __fastcall Packer::GetChapterPatterns(){
  return myChapterDividePatterns;
}

void __fastcall Packer::SetNormalFontImage(Graphics::TBitmap* value){
  myNormalFontImage = value;
}
Graphics::TBitmap* __fastcall Packer::GetNormalFontImage(){
  return myNormalFontImage;
}

void __fastcall Packer::SetRotatedFontImage(Graphics::TBitmap* value){
  myRotatedFontImage = value;
}
Graphics::TBitmap* __fastcall Packer::GetRotatedFontImage(){
  return myRotatedFontImage;
}

void __fastcall Packer::SetEncodingFile(AnsiString value)
{
  if(myEncodingFile != value) {
    if(!FileExists(value)) value = "";
    myEncodingFile = value;

    if(myEncodingFile != ""){
      int fIn = FileOpen(myEncodingFile, fmOpenRead | fmShareDenyNone);
      if(fIn < 0){
        TLogger::error(AnsiString("Cannot open  file ")+myEncodingFile);
      }else{
        FileRead(fIn, myEncodingTable, 258);
        FileClose(fIn);
      }
    }
  }
}
AnsiString __fastcall Packer::GetEncodingFile(){
  return myEncodingFile;
}


void __fastcall Packer::SetLocalizationFile(AnsiString value)
{
  if(myLocalizationFile != value) {
    myLocalizationFile = FileExists(value) ? value : AnsiString("");
  }
}
AnsiString __fastcall Packer::GetLocalizationFile()
{
  return myLocalizationFile;
}
