//---------------------------------------------------------------------------

#ifndef PackerH
#define PackerH
//---------------------------------------------------------------------------
#include <vcl.h>

#include "ZipBuilder.h"
#include "TOC.h"

typedef AnsiString String;

/***********************************
***********************************/
class Packer {

private:
  int myMaxBookSize, myMaxJarTextSize;
  TOC myTocBuilder;
  
  TZipBuilder* myZipBuilder;
  TStringList* myManifestContent;
  TStringList* myFilePaths, * myFileNames;

  TStringList* mySourceJarDescriptions, *mySourceJarVersions;

  TStringList* myCreatedJars;
  TStringList* myMidletProperties;
  AnsiString myProjectDir;
  AnsiString myTempPath;
  AnsiString myTemplateJarPath;
  int myLastJarNumber;
  
  AnsiString myChapterDividePatterns;
//  TStringList* myChapterDividePatterns;
//  char* myChapterDividePatterns;

  AnsiString myJarRepository;
  int mySelectedPlatformIndex;
  Graphics::TBitmap* myNormalFontImage;
  Graphics::TBitmap* myRotatedFontImage;
  AnsiString myEncodingFile;
  char* myEncodingTable;

  void cleanUp(AnsiString projName);

  // deprecated
  bool createJar(AnsiString suiteId, int& textIndex, int& textPosition);

  AnsiString createNextJar(AnsiString suiteId);
  
  bool addFileFragment(int numInJar, char* buffer, int from, int size, AnsiString fragmentLabel);
//  bool addFileFragment(int numInJar, AnsiString file, int from, int size);
  
  void readAvailableJars();
  void __fastcall SetPlatformId(AnsiString value);
  AnsiString __fastcall GetPlatformId();
  int __fastcall GetSupportedPlatformsCount();
  TStringList* readManifest(AnsiString jarFilePath, TStringList* manifest);
  void __fastcall SetPlatformIndex(int value);
  int __fastcall GetPlatformIndex();

//  int findNextChapter(char* fileText, int from, int fileSize);
  void __fastcall SetChapterPatterns(AnsiString value);
  AnsiString __fastcall GetChapterPatterns();
  void __fastcall SetNormalFontImage(Graphics::TBitmap* value);
  Graphics::TBitmap* __fastcall GetNormalFontImage();
  void __fastcall SetRotatedFontImage(Graphics::TBitmap* value);
  Graphics::TBitmap* __fastcall GetRotatedFontImage();
  void __fastcall SetEncodingFile(AnsiString value);
  AnsiString __fastcall GetEncodingFile();

  void closeJar(AnsiString jarFile);
  bool writeMidletProperties(AnsiString jarFile);
  int myDefaultMidletPropertiesCount;
  AnsiString myLocalizationFile;
  char* toUnicode(AnsiString nativeContents);
  void __fastcall SetLocalizationFile(AnsiString value);
  AnsiString __fastcall GetLocalizationFile();

public:
        Packer(TZipBuilder* theZipBuilder, AnsiString jarRepository);
        bool build(AnsiString dir, AnsiString projName, int maxBookSize, int maxJarTextSize);


        int addText(AnsiString filePath);
        void removeText(int index);
        int getTextCount();
        AnsiString getTextName(int index);
        void setTextName(int index, AnsiString newName);
        AnsiString getTextPath(int index);

        TStringList* getCreatedJars();

        AnsiString __fastcall GetSupportedPlatformVersion(int index);
        AnsiString __fastcall GetSupportedPlatformId(int index);
        AnsiString __fastcall GetSupportedPlatformDescription(int index);


  void setMidletProperty(AnsiString name, AnsiString value);
  AnsiString getMidletProperty(AnsiString name);
  void clearMidletProperties();

  AnsiString getManifestField(TStringList* manifest, AnsiString fieldName);
  void setManifestField(TStringList* manifest, AnsiString fieldName, AnsiString value);

  __property AnsiString PlatformId  = { read=GetPlatformId, write=SetPlatformId };
  __property int SupportedPlatformsCount  = { read=GetSupportedPlatformsCount};
  __property int PlatformIndex  = { read=GetPlatformIndex, write=SetPlatformIndex };
  __property AnsiString ChapterPatterns  = { read=GetChapterPatterns, write=SetChapterPatterns };
  __property Graphics::TBitmap* NormalFontImage  = { read=GetNormalFontImage, write=SetNormalFontImage };
  __property Graphics::TBitmap* RotatedFontImage  = { read=GetRotatedFontImage, write=SetRotatedFontImage };
  __property AnsiString EncodingFile  = { read=GetEncodingFile, write=SetEncodingFile };
  __property AnsiString LocalizationFile  = { read=GetLocalizationFile, write=SetLocalizationFile };
};
#endif

