//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SelectImageUnit.h"
#include "Images.h"
#include "Logger.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSelectFontImageForm *SelectFontImageForm;
//---------------------------------------------------------------------------
__fastcall TSelectFontImageForm::TSelectFontImageForm(TComponent* Owner)
  : TForm(Owner) {
  myFontImages = new TStringList();
  myFontNames = new TStringList();
  myFontImgBounds = NULL;

  myDir = "";
}
//---------------------------------------------------------------------------
void __fastcall TSelectFontImageForm::FormCreate(TObject *Sender){
  SelectedIndex = -1;
}

//---------------------------------------------------------------------------
void TSelectFontImageForm::refreshFonts(){
  if(myFontImgBounds != NULL) {
    delete[] myFontImgBounds;
  }
  // clear (remove) all the images
  for(int i = 0; i < myFontImages->Count; i++){
    TObject* bmp = myFontImages->Objects[i];
    if(bmp != NULL) delete bmp;
  }
  myFontImages->Clear();

  // read fonts from the dir
  Images::listImages(myFontImages, myDir, true);

  myFontNames->Clear();
  for(int i = 0; i < myFontImages->Count; i++){
    AnsiString imgPath = myFontImages->Strings[i];
    AnsiString imgName = ExtractFileName(imgPath);
    int dotPos = imgName.Pos(".");
    imgName = imgName.SubString(1, dotPos-1);
    myFontNames->Add(imgName);
  }

  if(SelectedIndex >= myFontImages->Count) {
    SelectedIndex = myFontImages->Count - 1;
  }
    
  myFontImgBounds = new TRect[myFontImages->Count];
  ScrollBox1Resize(NULL);
}

//---------------------------------------------------------------------------
bool TSelectFontImageForm::selectImage(){
  refreshFonts();
  if(SelectedIndex > -1) scrollToFont(SelectedIndex);
  return ShowModal() == mrOk;
}

//---------------------------------------------------------------------------
void __fastcall TSelectFontImageForm::ScrollBox1Resize(TObject *Sender){
  // when resized, set width of paintbox to width, and calculate heigth

  int areaWidth = ScrollBox1->Width - 20;
  int areaHeight = 0;
  int curRowWidth = 0, curRowHeight = 0;

  for(int i = 0; i < myFontImages->Count; i++){
    Graphics::TBitmap* bmp = dynamic_cast<Graphics::TBitmap*> (myFontImages->Objects[i]);
    int bmpWidth = bmp->Width;
    int strWidth = FontPaintBox->Canvas->TextWidth(myFontNames->Strings[i]);
    int w = (bmpWidth > strWidth) ? bmpWidth : strWidth;
    w+= 4;
    int h = 4 + bmp->Height + FontPaintBox->Canvas->TextHeight(myFontNames->Strings[i]);

    int x = curRowWidth, y = areaHeight;

    if(curRowWidth + w > areaWidth && curRowWidth > 0){
      areaHeight += curRowHeight;

      curRowHeight = h;                        
      curRowWidth = w;
      x = 0; y = areaHeight;
    } else { // increase current row
      if(curRowHeight < h){
        curRowHeight = h;
      }
      curRowWidth += w;
    }

    myFontImgBounds[i] = Bounds(x, y, w, h);
  }

  areaHeight += curRowHeight;
  FontPaintBox->Width = areaWidth;
  FontPaintBox->Height = areaHeight;
}

//---------------------------------------------------------------------------
void __fastcall TSelectFontImageForm::FontPaintBoxPaint(TObject *Sender){
  for(int i = 0; i < myFontImages->Count; i++){
    drawFont(i);
  }
}
//---------------------------------------------------------------------------

void TSelectFontImageForm::drawFont(int fontIndex) {
   int x = myFontImgBounds[fontIndex].Left, y = myFontImgBounds[fontIndex].Top;

   AnsiString name = myFontNames->Strings[fontIndex];
   if(fontIndex == SelectedIndex){
      TColor prevColor = FontPaintBox->Canvas->Brush->Color;
      FontPaintBox->Canvas->Brush->Color = clBlue;
      FontPaintBox->Canvas->FillRect(myFontImgBounds[fontIndex]);
      FontPaintBox->Canvas->Brush->Color = prevColor;
   }

   int textX = (myFontImgBounds[fontIndex].Width() - FontPaintBox->Canvas->TextWidth(name)) / 2;
   FontPaintBox->Canvas->TextOut(x+textX, y+2, name);

   Graphics::TBitmap* bmp = dynamic_cast<Graphics::TBitmap*> (myFontImages->Objects[fontIndex]);
   FontPaintBox->Canvas->Draw(x+2, y+2+FontPaintBox->Canvas->TextHeight(name), bmp);
}

//---------------------------------------------------------------------------
// creates B/W Bitmap with glyphs of the font
Graphics::TBitmap* TSelectFontImageForm::createFontImage(TFont* font, Graphics::TBitmap* useIfProvided){
  Graphics::TBitmap* fontBmp = (useIfProvided != NULL) ? useIfProvided : new Graphics::TBitmap();
  TCanvas *imgCanvas = fontBmp->Canvas;
  imgCanvas->Font = font;
  int charWidth = imgCanvas->TextWidth("a"), charHeight = imgCanvas->TextHeight("a");

  fontBmp->Width = charWidth * 16;
  fontBmp->Height = charHeight * 14;

  // write bitmap:
  TRect screenRect(0,0, fontBmp->Width, fontBmp->Height);
  imgCanvas->Brush->Color = clWhite;
  imgCanvas->FillRect(screenRect);
  imgCanvas->Pen->Color = clBlack;

  int yPos = 0, xPos = 0;

  for(int i = 32; i < 256; i++){
    TRect rc(xPos,yPos, xPos + charWidth, yPos + charHeight);
    imgCanvas->TextRect(rc, xPos, yPos, AnsiString(char(i)));

    xPos += charWidth;
    if((i % 16) == 15){
      xPos = 0;
      yPos += charHeight;
    }
  }

  return fontBmp;
}


//---------------------------------------------------------------------------
// creates a font in image dir, as B/W
void TSelectFontImageForm::saveSelectedFont(TFont* font){
    Graphics::TBitmap* fontBmp = new Graphics::TBitmap();
    bool isFixed = false;
    TEXTMETRIC tm;
    fontBmp->Canvas->Font = font;
    if(::GetTextMetrics(fontBmp->Canvas->Handle, &tm)){
      isFixed = !(tm.tmPitchAndFamily & TMPF_FIXED_PITCH);
    }

    if(!isFixed){
      ShowMessage(AnsiString("Can use only fixed fonts! (")+ font->Pitch +")");
      delete fontBmp;
      return;
    }

    int charWidth = fontBmp->Canvas->TextWidth("a"), charHeight = fontBmp->Canvas->TextHeight("a");

    AnsiString uniqueFontName = font->Name+"_"+ charWidth +"x"+ charHeight +".png";
    AnsiString prompt = FileExists(myDir + "\\" + uniqueFontName) ? "File already exists" : "Save font as...";
    bool saveFont = true;
    do {
      if(!InputQuery(prompt, "Enter new name", uniqueFontName)){ // cancel
        saveFont = false;
        break;
      }
      prompt = "File already exists";
    }while( FileExists(myDir + "\\" + uniqueFontName) );

    if(saveFont){
      createFontImage(font, fontBmp);
      AnsiString selPath = myDir + "\\" + uniqueFontName;
      Images::write(selPath, fontBmp);
      refreshFonts();
      SelectedImagePath = selPath;
      FontPaintBox->Repaint();
      scrollToFont(SelectedIndex);
    }

  delete fontBmp;
}

//---------------------------------------------------------------------------
void __fastcall TSelectFontImageForm::NoFontButtonClick(TObject *Sender){
  SelectedIndex = -1;
  ModalResult = mrOk;
}

//---------------------------------------------------------------------------
void TSelectFontImageForm::scrollToFont(int index){
  if(index >= 0){
    ScrollBox1->VertScrollBar->Position = myFontImgBounds[index].Top;
  }
}

void __fastcall TSelectFontImageForm::SetSelectedImagePath(AnsiString value){
  // look for image with same path, then set sel index to this index
  for(int i = 0; i < myFontImages->Count; i++){
    if(myFontImages->Strings[i] == value){
      SelectedIndex = i;
      return;
    }
  }
  SelectedIndex = -1;
}

AnsiString __fastcall TSelectFontImageForm::GetSelectedImagePath(){
  if (SelectedIndex < 0 || SelectedIndex >= myFontImages->Count) return "";
  return myFontImages->Strings[SelectedIndex];
}

//--------------------------------------------------------
Graphics::TBitmap* __fastcall TSelectFontImageForm::GetSelectedImage()
{
  if (SelectedIndex < 0 || SelectedIndex >= myFontImages->Count) return NULL;
  return dynamic_cast<Graphics::TBitmap*> (myFontImages->Objects[SelectedIndex]);
}

void __fastcall TSelectFontImageForm::SetSelectedIndex(int value){
    mySelectedImageIndex = value;
    if(mySelectedImageIndex < 0 || mySelectedImageIndex >= myFontImages->Count){
      mySelectedImageIndex = -1;
    }
    RemoveFontButton->Enabled = (mySelectedImageIndex >= 0);
    // SelFontButton->Enabled = (mySelectedImageIndex >= 0);
    _SubmitBtn->Enabled = (mySelectedImageIndex >= 0);
}

int __fastcall TSelectFontImageForm::GetSelectedIndex(){
  return mySelectedImageIndex;
}

void __fastcall TSelectFontImageForm::FontPaintBoxMouseDown(
      TObject *Sender, TMouseButton Button, TShiftState Shift, int X,
      int Y){
//
  int selFontIndex = -1;

  for(int i=0; i < myFontImages->Count; i++){
    TRect rect = myFontImgBounds[i];
    if(X >= rect.Left && Y >= rect.Top && X < rect.Right && Y < rect.Bottom){
      selFontIndex = i;
      break;
    }
  }

  if(selFontIndex != SelectedIndex){
    SelectedIndex = selFontIndex;
    FontPaintBox->Repaint();
  }
}
//---------------------------------------------------------------------------

void __fastcall TSelectFontImageForm::FontPaintBoxDblClick(TObject *Sender)
{
// select image
  if(SelectedIndex >= 0){ //
    ModalResult = mrOk;
  }
}
//---------------------------------------------------------------------------


void __fastcall TSelectFontImageForm::FormShortCut(TWMKey &Msg,
      bool &Handled)
{
  if(Msg.CharCode == VK_CANCEL || Msg.CharCode == 27){
    ModalResult = mrCancel;
    Handled = true;
  }

}
//---------------------------------------------------------------------------


void __fastcall TSelectFontImageForm::SetDirectory(AnsiString value)
{
  if(myDir != value) {
    myDir = value;
    SelectedIndex = -1;
    refreshFonts();
  }
}
AnsiString __fastcall TSelectFontImageForm::GetDirectory()
{
  return myDir;
}
void __fastcall TSelectFontImageForm::FontPaintBoxMouseMove(
      TObject *Sender, TShiftState Shift, int X, int Y){

  bool overFont = false;
  for(int i=0; !overFont && i < myFontImages->Count; i++){
    TRect rect = myFontImgBounds[i];
    if(X >= rect.Left && Y >= rect.Top && X < rect.Right && Y < rect.Bottom){
      overFont = true;
    }
  }
  if(overFont != (FontPaintBox->Cursor != crDefault)){
    FontPaintBox->Cursor = overFont ? crHandPoint : crDefault;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSelectFontImageForm::AddFontBtnClick(TObject *Sender){
  FontDialog1->Options.Clear();
  FontDialog1->Options << fdFixedPitchOnly;
  if(FontDialog1->Execute()){
    saveSelectedFont(FontDialog1->Font);// creates bw font
  }
}
//---------------------------------------------------------------------------

void __fastcall TSelectFontImageForm::RemoveFontButtonClick(
      TObject *Sender){
  if(SelectedIndex >= 0){
    DeleteFile(myFontImages->Strings[SelectedIndex]);
    refreshFonts();
    FontPaintBox->Repaint();
    scrollToFont(SelectedIndex);
  }
}
//---------------------------------------------------------------------------

