//---------------------------------------------------------------------------
// #include <vector>


#pragma hdrstop

#include "TOC.h"
#include "Logger.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

//---------------------------------------------------------------------------
TOC::TOC(){
  myText = NULL;
  myTextSize = 0;
  myTitles = new TStringList();
}

//---------------------------------------------------------------------------
void TOC::build(char* text, int size, char* chapterDividePatterns, bool splitByNumIndex){
  myText = text;
  myTextSize = size;

  myPositions.clear();
  myTitles->Clear();

  if(!chapterDividePatterns || !*chapterDividePatterns) {
     return;
  }

  int pos = 0;

  while(pos < myTextSize){
    int lineStart = pos;

/*
    while(pos < myTextSize && (myText[pos] == ' ' || myText[pos] == '\t')) pos++;
    if(pos >= myTextSize){ break;}
    if(myText[pos] == '\n') {pos++; continue; } // empty line
*/

    unsigned char c;
    while(pos < myTextSize && (c = myText[pos]) <= 32) {
      pos++;
      if(c == '\n') lineStart = pos;
    }
    if(pos >= myTextSize){ break;}

    bool isChapterStart = (lineStart == 0);

    char* patterns = chapterDividePatterns;
    while(*patterns && !isChapterStart){ // while there are patterns to compare
      // check cur against fileText[pos]
      int p = pos;
      while(p < myTextSize && *patterns && *patterns != ',' && myText[p]==*patterns){
        p++; patterns++;
      }

      if(!*patterns || *patterns == ','){ // found !!!
        // char* dbg = myText + lineStart;
        //push(lineStart);
        isChapterStart = true;
        break;
      }

      // does not match, skip to next pattern
      while(*patterns && *patterns != ',') patterns++;
      if(*patterns == ',')patterns++;
    }

    int chapTitleStart = pos;
    // look for next BOL
    while(pos < myTextSize && myText[pos] != '\n') pos++;

    if(isChapterStart){
      myPositions.push_back(lineStart);
      AnsiString chaptitle = AnsiString(myText + chapTitleStart, pos - chapTitleStart);
      myTitles->Add(chaptitle);
//      TLogger::log(AnsiString("Add chapter: ")+chaptitle);
    }
    pos++;
  }
}

//---------------------------------------------------------------------------
int TOC::chapterCount(){
  return myPositions.size();
}

//---------------------------------------------------------------------------
int TOC::getNext(int from){
  if(chapterCount() == 0)
    return myTextSize;

  // look for next line, and find nearest bigger BOL
  int pos = from;
  while(pos < myTextSize && myText[pos] != '\n') pos++;
  pos++;
  if(pos >= myTextSize) return myTextSize;
  for(int i=0, len = myPositions.size(); i < len; i++){
    if(myPositions[i] >= pos)
      return myPositions[i];
  }

  return myTextSize;
}

//---------------------------------------------------------------------------
TOC::~TOC(){
  delete myTitles;
}

void TOC::join(int fromChapter, int toChapterInclusive){
  //TODO: Add your source code here
}

AnsiString TOC::getChapterTitle(int chapter){
  return myTitles->Strings[chapter];
}

void TOC::setChapterTitle(int chapter, AnsiString newTitle){
  myTitles->Strings[chapter] = newTitle;
}
