//---------------------------------------------------------------------------

#define NO_WIN32_LEAN_AND_MEAN
#include <shlobj.hpp>

#include <dir.h>
#include <vcl.h>

#include <filectrl.hpp>
#include <StrUtils.hpp>

#include <Registry.hpp>
#pragma hdrstop

#include "Unit1.h"
#include "Logger.h"
#include "Localizer.h"
#include "Utils.h"
#include "Packer.h"
#include "ResultForm.h"

#include "Images.h"
#include "SelectImageUnit.h"

#include <strstream>


// good one
//#define MSG(x) {strstream _tmp_c_str; _tmp_c_str<<x; ShowMessage(_tmp_c_str.str());}

#define STR(x) ((strstream&)(strstream()<<""<<x)).str()
#define MSG(x) ShowMessage(STR(x))


//--------------------------------w-------------------------------------------
#pragma package(smart_init)
#pragma link "ZipBuilder"
#pragma resource "*.dfm"
TShellForm *ShellForm;
static AnsiString APP_VER = "1.6";
static AnsiString PROGRAM_REGISTRY_KEY = "Software\\TCBR Shell\\v" + APP_VER;

//---------------------------------------------------------------------------
__fastcall TShellForm::TShellForm(TComponent* Owner): TForm(Owner){
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::FormCreate(TObject *Sender){
  //ShowMessage(((strstream&)(strstream()<<(char*)"aaa,"<<"bbb"<<endl<<"ccc")).str());
///  ShowMessage((strstream() ).str());

  // MSG("aaa,"<<"bbb"<<endl<<"ccc");
  //ShowMessage(strstream().str());
  
/*
    options:
      Phone has dir buttons (def - true)
      Override height by 16 (Motorola/Samsung)
      Use PNG font when searching
*/
  TLogger::setDisplay(myLogText);
  TLogger::clearLog();
  TLocalizer::localize(this);

  SelectFontImageForm = new TSelectFontImageForm(this);
  SelectFontImageForm->Directory = ExtractFileDir(Application->ExeName) + "\\fonts";
  TLocalizer::localize(SelectFontImageForm);

  Memo1->Lines->LoadFromFile(TLocalizer::getLocalFile("readme.txt"));

  myNormalFontImage = new Graphics::TBitmap();
  myRotatedFontImage = new Graphics::TBitmap();

  ///////////////// READ REGISTRY
  // set last remembered colors,
  // prepare bitmaps from last remembered paths

  TRegistry *registry = new TRegistry(KEY_READ);
  registry->OpenKey(PROGRAM_REGISTRY_KEY,true);

  myBgColor = registry->ReadString("BgColor").ToIntDef(clWhite);
  myFgColor = registry->ReadString("FgColor").ToIntDef(clBlack);

  myNormalFontPath = registry->ReadString("NormalFontPath");
  myRotatedFontPath = registry->ReadString("RotatedFontPath");
  myOutputDir = registry->ReadString("OutDir");

  AnsiString maxTextSize = registry->ReadString("MaxTextSize");
  if(maxTextSize == "") maxTextSize = "0";
  myMaxBookSize->Text = maxTextSize;

  AnsiString maxJarSize = registry->ReadString("MaxJarSize");
  if(maxJarSize == "") maxJarSize = "0";
  myMaxSuiteTextSize->Text = maxJarSize;

  AnsiString platformId = registry->ReadString("platformId");
  AnsiString encoding = registry->ReadString("encoding");

  UseSeT610SideKeysCheckBox->Checked = registry->ReadString("sevolumekeys") == "true";
  AltDirButtonsCheckBox->Checked = registry->ReadString("altdir")== "true";

  RestrictChapterCheckBox->Checked = registry->ReadString("restricttext") == "true";
  RestrictJarCheckBox->Checked = registry->ReadString("restrictjar") == "true";

  LocalizeMidletCheckBox->Checked = registry->ReadString("localizeJar") == "true";

  myMaxBookSize->Enabled = RestrictChapterCheckBox->Checked;
  myMaxSuiteTextSize->Enabled = RestrictJarCheckBox->Checked;

  AnsiString dividers = registry->ReadString("chapterDividers");
  if(dividers != ""){
    myChapterDividePatterns->Text = dividers;
  }
  bool useChapterDividers = registry->ReadString("useChapterDividers") != "false";
  DivideChaptersCheckBox->Checked = useChapterDividers;
  myChapterDividePatterns->Enabled = useChapterDividers;

  ExtendHeightCheckBox->Checked = registry->ReadString("ExtendScreenHeight") == "true";
  ExtendHeightEdit->Enabled = ExtendHeightCheckBox->Checked;
  AnsiString extendBy = registry->ReadString("ExtendScreenHeightBy");
  if(extendBy == "") extendBy = "16";
  ExtendHeightEdit->Text = extendBy;

  HasDirButtonsCheckBox->Checked = registry->ReadString("PhoneHasDirButtons") != "false";

  delete registry;

  readEncodings(encoding);

  Graphics::TBitmap* tmpBmp = new Graphics::TBitmap();
  if(Images::read(tmpBmp, myNormalFontPath)){
    setFontImage(myNormalFontImage, tmpBmp);
  }else{
    setFontImage(myNormalFontImage, NULL);
  }
  if(Images::read(tmpBmp, myRotatedFontPath)){
    setFontImage(myRotatedFontImage, tmpBmp);
  }else{
    setFontImage(myRotatedFontImage, NULL);
  }

  delete tmpBmp;

  myPacker = new Packer(myZipBuilder, ExtractFileDir(Application->ExeName) + "\\jars");

  if(myPacker->SupportedPlatformsCount == 0){
    ShowMessage("No midlets in 'jars' subdirectory\ndownload them and save there");
    Application->Terminate();
  }

  MidletTypeCombo->Clear();
  for(int i=0; i < myPacker->SupportedPlatformsCount; i++){
    MidletTypeCombo->AddItem(myPacker->GetSupportedPlatformDescription(i), NULL);
  }
  myPacker->PlatformId = platformId;
  MidletTypeCombo->ItemIndex = myPacker->PlatformIndex;
  checkLocalizationAvailable();

  OleInitialize(NULL);

  //Caption = Application->Title;
//  Application->Title = Caption;
//  Caption = AnsiString("BookReader shell. Midlet Version: ")+myPacker->getMidletVersion();

  textFileList->Cells[0][0] = "Name";
  textFileList->Cells[1][0] = "Path";
  textFileList->Cells[2][0] = "Size";

  int height = textFileList->Font->Height;
  TEXTMETRIC tm;
  if(::GetTextMetrics(textFileList->Canvas->Handle, &tm)){
    height = tm.tmHeight + tm.tmExternalLeading + 5; //  + tm.tmAscent + tm.tmDescent;
  }
  textFileList->DefaultRowHeight = height;

  myZipBuilder->Load_Zip_Dll();
  myZipBuilder->Load_Unz_Dll();

//  myPacker->addText(ExtractFileDir(Application->ExeName) + "\\" + "test-texts\\pirog.txt");
  myPacker->addText(ExtractFileDir(Application->ExeName) + "\\" + "test-texts\\h2g2.txt");
//  myPacker->addText(ExtractFileDir(Application->ExeName) + "\\" + "test-texts\\Untitled1-SavedByTC.sql");
  updateList();

  if(!DirectoryExists(myOutputDir)){
    myOutputDir = ExtractFileDir(Application->ExeName);
    if(ExtractFileName(Application->ExeName) != "Project1.exe"){
      char* path = new char[MAXPATH];
      if(SHGetSpecialFolderPath(Handle, path, CSIDL_PERSONAL, false)){
        myOutputDir = path;
      }
      PageControl1->ActivePage = aboutTabSheet;
    }
  }

  Caption = Caption + " TequilaCat BookReader v"+APP_VER;
  Application->Title = Caption;
}

/* This call causes the callback to execute for every locale */


//---------------------------------------------------------------------------
void __fastcall TShellForm::FormDestroy(TObject *Sender){
  // Write params into registry
  TRegistry *registry = new TRegistry();
  registry->OpenKey(PROGRAM_REGISTRY_KEY,true);

  registry->WriteString("platformId", myPacker->PlatformId);

  registry->WriteString("localizeJar", LocalizeMidletCheckBox->Checked ? "true":"false");

  registry->WriteString("MaxTextSize", myMaxBookSize->Text);
  registry->WriteString("MaxJarSize", myMaxSuiteTextSize->Text);

  registry->WriteString("BgColor", myBgColor);
  registry->WriteString("FgColor", myFgColor);

  registry->WriteString("OutDir", myOutputDir);
  registry->WriteString("encoding", EncodingComboBox->Items->Strings[EncodingComboBox->ItemIndex]);

  registry->WriteString("chapterDividers", myChapterDividePatterns->Text);
  registry->WriteString("useChapterDividers", DivideChaptersCheckBox->Checked ? "true" : "false");

  registry->WriteString("NormalFontPath", myNormalFontPath);
  registry->WriteString("RotatedFontPath", myRotatedFontPath);

  registry->WriteString("ExtendScreenHeight", ExtendHeightCheckBox->Checked ? "true" : "false");
  registry->WriteString("ExtendScreenHeightBy", ExtendHeightEdit->Text);
  registry->WriteString("PhoneHasDirButtons", HasDirButtonsCheckBox->Checked ? "true" : "false");

  registry->WriteString("sevolumekeys", UseSeT610SideKeysCheckBox->Checked ? "true" : "false");
  registry->WriteString("altdir", AltDirButtonsCheckBox->Checked ? "true" : "false");
  registry->WriteString("restricttext", RestrictChapterCheckBox->Checked ? "true" : "false");
  registry->WriteString("restrictjar", RestrictJarCheckBox->Checked ? "true" : "false");

  registry->CloseKey();
  delete registry;

  myZipBuilder->Unload_Zip_Dll();
  myZipBuilder->Unload_Unz_Dll();
  OleUninitialize();
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::buildButtonClick(TObject *Sender){
  char* name = strdup(projectName->Text.c_str());
  for(int i = 0; name[i]; i++){
    if((unsigned char)name[i] > 127){
      ShowMessage(AnsiString("Project name: '")+ projectName->Text+"'\nOnly latinic letters please");
      return;
    }
  }

  int maxBookSize = RestrictChapterCheckBox->Checked ? myMaxBookSize->Text.ToIntDef(0) : 0;

  int maxJarTextSize = RestrictJarCheckBox->Checked ? myMaxSuiteTextSize->Text.ToIntDef(0) : 0;
  
  if(maxBookSize < 0 || maxJarTextSize < 0){
    ShowMessage(LNG("LNG_BAD_TEXT_SIZES", "Wrong text sizes"));
  }

  myPacker->ChapterPatterns = myChapterDividePatterns->Text;
  PageControl1->ActivePage = myLogTabSheet;
  Cursor = crHourGlass;

  myPacker->NormalFontImage = (myNormalFontPath == "") ? NULL: myNormalFontImage;
  myPacker->RotatedFontImage = (myRotatedFontPath == "") ? NULL: myRotatedFontImage;

  myPacker->EncodingFile = ExtractFileDir(Application->ExeName)+"\\i18n\\"+
    EncodingComboBox->Items->Strings[EncodingComboBox->ItemIndex] +".enc";

  myPacker->ChapterPatterns = DivideChaptersCheckBox->Checked ?
    myChapterDividePatterns->Text : AnsiString("");

  myPacker->clearMidletProperties();

  myPacker->setMidletProperty("bgcolor", AnsiString((GetRValue(myBgColor)<<16) | (GetGValue(myBgColor)<<8) | GetBValue(myBgColor)));
  myPacker->setMidletProperty("fgcolor", AnsiString((GetRValue(myFgColor)<<16) | (GetGValue(myFgColor)<<8) | GetBValue(myFgColor)));

  if(HasDirButtonsCheckBox->Checked){
    myPacker->setMidletProperty("hasdirbuttons", "true");
  }
  if(ExtendHeightCheckBox->Checked){
    int value = ExtendHeightEdit->Text.ToIntDef(0);
    if(value > 0){
      myPacker->setMidletProperty("plusheight", ExtendHeightEdit->Text);
    }
  }

  if(UseSeT610SideKeysCheckBox->Checked){
      myPacker->setMidletProperty("sevolumekeys", "true");
  }
  if(AltDirButtonsCheckBox->Checked){
      myPacker->setMidletProperty("altdir", "true");
  }
  
  myPacker->LocalizationFile = checkLocalizationAvailable();
  bool result = myPacker->build(myOutputDir, projectName->Text, maxBookSize, maxJarTextSize);

  Cursor = crDefault;

  if(!result){
     MessageDlg("JAR Building failed. Refer to log for details",
        mtError, TMsgDlgButtons() << mbAbort, 0);
  }else{
    TStrings* names = myPacker->getCreatedJars();
    TLogger::logHeader("Success! generated files: ", false);
    for(int i = 0; i < names->Count; i++){
      TLogger::message(names->Strings[i]);
    }
  }

}

// click myBrowseTargetDirBtn - open select folder button
// click underscored target - open directory
//---------------------------------------------------------------------------
void __fastcall TShellForm::_myBrowseTargetDirBtnClick(TObject *Sender){
  WideString fromPath = myOutputDir;
  if(SelectDirectory("Select output directory", WideString("\\"), myOutputDir)){
    myTargetDirLabel->Caption = MinimizeName(myOutputDir, myTargetDirLabel->Canvas, myTargetDirLabel->Width);
  }
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::myTargetDirLabelClick(TObject *Sender){
  ShellExecute(Handle, NULL, myOutputDir.c_str(), NULL, NULL, SW_SHOWDEFAULT	);
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::textFileListDblClick(TObject *Sender){
  shellEditText();
}
//---------------------------------------------------------------------------
void TShellForm::shellEditText(){
  int index = textFileList->Row - 1;
  if(index >= 0 && index < myPacker->getTextCount()){
    ShellExecute(Handle, NULL, myPacker->getTextPath(index).c_str(), NULL, NULL, SW_SHOWDEFAULT	);
  }
}

//---------------------------------------------------------------------------
void TShellForm::updateList(){
  //TODO: Add your source code here
  //int lastCol = textFileList->Col, lastRow = textFileList->Row;
  int textsNum = myPacker->getTextCount();

  if(textsNum == 0){
    textFileList->Cells[0][1] = "";
    textFileList->Cells[1][1] = "";
    textFileList->Cells[2][1] = "";
    textFileList->RowCount = 2;
  }else{
    textFileList->RowCount = textsNum + 1;
    for(int i=0; i < textsNum; i++){
      AnsiString path = myPacker->getTextPath(i);
      textFileList->Cells[0][i+1] = myPacker->getTextName(i);
      textFileList->Cells[1][i+1] = path;

      //textFileList->Cells[2][i+1] = AnsiString("")+ FileSizeByName(path);
      textFileList->Cells[2][i+1] = AnsiString("")+ Utils::getFileSize(path);
    }
  }

  myTBRemoveTxt->Enabled = (textsNum > 0);
  myTBEditTxt->Enabled = (textsNum > 0);
  myTBRenameTxt->Enabled = (textsNum > 0);

  buildButton->Enabled = (textsNum > 0);
}

void __fastcall TShellForm::FormResize(TObject *Sender){
// adjust size of the path column
  textFileList->ColWidths[1] = textFileList->Width - textFileList->ColWidths[0] - textFileList->ColWidths[2] - 10;

  myTargetDirLabel->Caption = MinimizeName(myOutputDir, myTargetDirLabel->Canvas, myTargetDirLabel->Width);
}
//---------------------------------------------------------------------------


void TShellForm::renameText(){
  int index = textFileList->Row - 1;
  if(index >= 0 && index < myPacker->getTextCount()){
    AnsiString result = InputBox("Rename Book", "Book name:", myPacker->getTextName(index));
    if(result != myPacker->getTextName(index)){
      myPacker->setTextName(index, result);
      updateList();
    }
  }
}
//---------------------------------------------------------------------------

void TShellForm::removeText(){
  int index = textFileList->Row - 1;
  if(index >= 0 && index < myPacker->getTextCount()){
    myPacker->removeText(index);
    updateList();
  }
}

//---------------------------------------------------------------------------
void TShellForm::addText(){
  textFileOpenDialog->Filter = "Text files (*.txt)|*.TXT|All files (*.*)|*.*";
  if(textFileOpenDialog->Execute()){
    int curLen =myPacker->getTextCount();
    if(curLen < myPacker->addText(textFileOpenDialog->FileName)){
      updateList();
    }
  }
}

//---------------------------------------------------------------------------

void __fastcall TShellForm::myTBAddTxtClick(TObject *Sender){
  addText();
}

void __fastcall TShellForm::myTBRenameTxtClick(TObject *Sender){
  renameText();
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::myTBEditTxtClick(TObject *Sender){
  shellEditText();
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::myTBRemoveTxtClick(TObject *Sender){
  removeText();
}
//---------------------------------------------------------------------------
void __fastcall TShellForm::textFileListKeyDown(TObject *Sender, WORD &Key, TShiftState Shift){
    if(Key == VK_DELETE){
      removeText();
    }else
    if(Key == VK_INSERT){
      addText();
    }else
    if(Key == VK_F2){
      renameText();
    }
}


//---------------------------------------------------------------------------
void __fastcall TShellForm::myClearLogButtonClick(TObject *Sender){
  TLogger::clearLog();
}

/**************************************************************************

Font Works

**************************************************************************/


//---------------------------------------------------------------------------

void __fastcall TShellForm::myTargetDirLabelMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y){
//
  if(Button == mbRight){
    Utils::showContextPopupMenu(myOutputDir, Handle, TPoint(X, Y), Shift.Contains(ssShift));
  }
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::myZipBuilderMessage(TObject *Sender,
      int ErrCode, AnsiString Message){
  if(ErrCode != 0 || myCBShowZipDebug->Checked){
    AnsiString msg = Message + " (error = "+ErrCode+")";
    if(ErrCode == 3) msg += "  Please replace data\\bookreader.jar";
    if(ErrCode > 0)
      TLogger::error(msg);
    else
      TLogger::log(msg);
  }
}

//---------------------------------------------------------------------------
void TShellForm::setFontImage(Graphics::TBitmap* bmp, Graphics::TBitmap* fromImage){
  // loads from appropriate path, if path exists
  // then changes all the colors
//  AnsiString& path = (bmp == myNormalFontImage) ? myNormalFontPath:myRotatedFontPath;
//  if(Images::read(bmp, path)){
  if(fromImage != NULL){
    bmp->Width = fromImage->Width;
    bmp->Height = fromImage->Height;
    bmp->Canvas->Draw(0,0, fromImage);
    setImageColors(bmp);
  }else{
    AnsiString usePhoneFont = ((TButton*)SelectFontImageForm->FindComponent("NoFontButton"))->Caption;

    bmp->Canvas->Brush->Color = myBgColor;
    bmp->Canvas->Font->Color = myFgColor;
    bmp->Canvas->Font->Style = bmp->Canvas->Font->Style << fsBold;

    bmp->Width = bmp->Canvas->TextWidth(usePhoneFont);
    bmp->Height = bmp->Width;

    bmp->Canvas->FillRect(Bounds(0,0,bmp->Width, bmp->Height));
    bmp->Canvas->TextOut(0, (bmp->Height-bmp->Canvas->TextWidth("A")) / 2, usePhoneFont);
  }
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::FontImagePaintBoxPaint(TObject *Sender){
// paint font images:
  int imgTop = 5 + SelNormalFontTButton->Top + SelNormalFontTButton->Height - FontImagePaintBox->Top;
  int leftOffset = 4;
  
  FontImagePaintBox->Canvas->Draw(leftOffset, imgTop, myNormalFontImage);
  FontImagePaintBox->Canvas->Draw(leftOffset + FontImagePaintBox->Width/2, imgTop, myRotatedFontImage);
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::SelectRotatedFontBtnClick(TObject *Sender){
  SelectFontImageForm->SelectedImagePath = myRotatedFontPath;
  if(SelectFontImageForm->selectImage()){ // ExtractFileDir(Application->ExeName) + "\\fonts");
    //ShowMessage(AnsiString("Rotated font selected: ") + (int)SelectFontImageForm->SelectedImage);
    myRotatedFontPath = SelectFontImageForm->SelectedImagePath;
    setFontImage(myRotatedFontImage, SelectFontImageForm->SelectedImage);
    FontImagePaintBox->Repaint();
  }
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::SelectNormalFontBtnClick(TObject *Sender){
//  SelectFontImageForm->selectFontImage(ExtractFileDir(Application->ExeName) + "\\fonts");
  SelectFontImageForm->SelectedImagePath = myNormalFontPath;
  if(SelectFontImageForm->selectImage()){ // ExtractFileDir(Application->ExeName) + "\\fonts");
    //ShowMessage(AnsiString("Normal font selected: ") + (int)SelectFontImageForm->SelectedImage);
    myNormalFontPath = SelectFontImageForm->SelectedImagePath;
    setFontImage(myNormalFontImage, SelectFontImageForm->SelectedImage);
    FontImagePaintBox->Repaint();
  }
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::SelFGColorBtnClick(TObject *Sender){
  selectColor(myFgColor);
}
//---------------------------------------------------------------------------
void __fastcall TShellForm::SelBGColorBtnClick(TObject *Sender){
  selectColor(myBgColor);
}

//---------------------------------------------------------------------------
void TShellForm::selectColor(TColor& color){
// sel color
  ColorDialog1->Color = color;
  if(ColorDialog1->Execute()){
    color = ColorDialog1->Color;
    setImageColors(myNormalFontImage);
    setImageColors(myRotatedFontImage);
    FontImagePaintBox->Repaint();
  }
}

//---------------------------------------------------------------------------
void TShellForm::setImageColors(Graphics::TBitmap* bmp){
  TColor oldBgColor = bmp->Canvas->Pixels[0][0];
  int w = bmp->Width, h = bmp->Height;
  for(int x = 0; x < w; x++){
    for(int y = 0; y < h; y++){
      TColor col = bmp->Canvas->Pixels[x][y];
      bmp->Canvas->Pixels[x][y] = (col == oldBgColor) ? myBgColor:myFgColor;
    }
  }
}

//---------------------------------------------------------------------------
void __fastcall TShellForm::MidletTypeComboSelect(TObject *Sender){
  //ShowMessage(AnsiString("Selected Item: ")+MidletTypeCombo->ItemIndex);
  myPacker->PlatformIndex = MidletTypeCombo->ItemIndex;
  checkLocalizationAvailable();
}


void __fastcall TShellForm::PageControl1Resize(TObject *Sender){
  SelRotatedFontTButton->Left = PageControl1->Width / 2;
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::InvertFontButtonClick(TObject *Sender){
  //
  TColor tmpColor = myBgColor;
  myBgColor = myFgColor;
  myFgColor = tmpColor;
  setImageColors(myNormalFontImage);
  setImageColors(myRotatedFontImage);
  FontImagePaintBox->Repaint();
}
//---------------------------------------------------------------------------
void TShellForm::readEncodings(AnsiString defaultEncodingName){
  EncodingComboBox->Items->Add("Western European");
  EncodingComboBox->ItemIndex = 0;

  TSearchRec sr;
  if(FindFirst(ExtractFileDir(Application->ExeName)+"\\i18n\\*.enc", 0, sr) == 0){
    do{
      AnsiString encName = sr.Name.SubString(1, sr.Name.Length() - 4);
      EncodingComboBox->Items->Add(encName);
      if(encName == defaultEncodingName){
        EncodingComboBox->ItemIndex = EncodingComboBox->Items->Count-1;
      }
    } while (FindNext(sr) == 0);
  }
  FindClose(sr);
}
void __fastcall TShellForm::DivideChaptersCheckBoxClick(TObject *Sender){
//
  myChapterDividePatterns->Enabled = DivideChaptersCheckBox->Checked;
}
//---------------------------------------------------------------------------


void __fastcall TShellForm::ExtendHeightCheckBoxClick(TObject *Sender){
  ExtendHeightEdit->Enabled = ExtendHeightCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::RestrictChapterCheckBoxClick(TObject *Sender){
  myMaxBookSize->Enabled = RestrictChapterCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::RestrictJarCheckBoxClick(TObject *Sender){
  myMaxSuiteTextSize->Enabled = RestrictJarCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TShellForm::Image1DblClick(TObject *Sender)
{
  int selStart = Memo1->SelStart;
  Memo1->Lines->LoadFromFile(TLocalizer::getLocalFile("readme.txt"));
  Memo1->SelStart = selStart;
}
//---------------------------------------------------------------------------
// depending on existance of loc file, and on chekcedness of da checkbox
AnsiString TShellForm::checkLocalizationAvailable(){
  AnsiString name = AnsiString("midletmsg-")+myPacker->GetSupportedPlatformVersion(MidletTypeCombo->ItemIndex)+".txt";
  AnsiString midletLocalization = TLocalizer::getLocalFile(name);

  if(FileExists(midletLocalization)){
    LocalizeMidletCheckBox->Enabled = true;
    return LocalizeMidletCheckBox->Checked ? midletLocalization : AnsiString("");
  }

  LocalizeMidletCheckBox->Enabled = false;
  LocalizeMidletCheckBox->Checked = false;
  return "";
}
